import { Injectable } from '@angular/core';
import { Spell } from 'src/interfaces/spell';
import { Subject } from 'rxjs';

@Injectable({
  providedIn: 'root',
})
export class SpellsService {
  // Functions

  private _closeSubject = new Subject<number>();

  public closeSubject$ = this._closeSubject.asObservable();

  constructor() {}

  public closeAllOthers(level: number): void {
    this._closeSubject.next(level);
  }

  // Custom Spells

  public customSpells: Spell[] = [];

  /**
   * Returns an array of available spells for the given level and character class.
   * The results come from the official apells array and the custom spells array.
   * @param level The level of the spell.
   * @param characterClass The character class of the spell.
   * @returns An array of available spells.
   */
  public getAvailableSpells(level: number, characterClass: string): Spell[] {
    let result = this.spells
      .filter((spell) => spell.level === level)
      .filter((spell) => spell.classes.includes(characterClass));
    result.push(...this.customSpells.filter((spell) => spell.level === level));
    console.log(result);

    return result;
  }

  /**
   * Deletets a single spell from the custom spells array and the full spells array.
   * @param spellToDelete The spell to be deleted from the custom spells.
   */
  public deleteCustomSpell(spellToDelete: Spell): void {
    const customIndex = this.customSpells.findIndex(
      (spell) => spell.id === spellToDelete.id
    );
    if (customIndex > -1) {
      this.customSpells.splice(customIndex, 1);
    }
    const fullIndex = this.spells.findIndex(
      (spell) => spell.id === spellToDelete.id
    );
    if (fullIndex > -1) {
      this.spells.splice(fullIndex, 1);
    }
  }

  //

  private spells: Spell[] = [
    // Level 0
    {
      id: 0,
      isCustom: false,
      german: 'Göttliche Führung',
      english: 'guidance',
      classes: ['Cleric', 'Druid'],
      level: 0,
      cost: 'action',
      duration: 10,
      timeToCast: 0,
      canRitual: 'false',
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: true,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Du berührst eine bereitwillige Kreatur. Einmal vor dem Ende des Zaubers kann das Ziel mit einem W4 würfeln und das Ergebnis zu einem Attributswurf seiner Wahl addieren. Es kann mit dem W4 vor oder nach dem Attributswurf würfeln. Dann endet der Zauber.',
      school: 'Divination',
      isRanged: false,
      range: 5,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      id: 1,
      isCustom: false,
      german: 'Totenläuten',
      english: 'tollTheDead',
      classes: ['Cleric', 'Warlock', 'Wizard'],
      level: 0,
      cost: 'action',
      duration: 0,
      timeToCast: 0,
      canRitual: 'false',
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: false,
      needsAttackRoll: false,
      needsSavingThrow: true,
      savingThrowAttribute: 'wisdom',
      doesDamage: true,
      damage: [{ diceNumber: '1', diceType: 'd8', damageType: 'necrotic' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Du zeigst auf eine Kreatur innehalb von 18 Meter, die du sehen kannst und der Klang schmerzhafter Glocken füllt die Luft um sie herum für einen Moment. Das Ziel muss einen Weisheitsrettungswurf machen oder 1W8 Nekrotischen Schaden erleiden. Wenn das Ziel bereits Schaden erlitten hat, erhöht sich der Schaden auf 1W12.',
      school: 'Necromancy',
      isRanged: true,
      range: 60,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      id: 2,
      isCustom: false,
      german: 'Thaumaturgie',
      english: 'thaumaturgy',
      classes: ['Cleric'],
      level: 0,
      cost: 'action',
      duration: 10,
      timeToCast: 0,
      canRitual: 'false',
      needsVerbal: true,
      needsSomatic: false,
      needsMaterial: false,
      needsConcentration: false,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Du erzeugst einen der folgenden magischen Effekte innerhalb von 9 Metern: \b\b • Deine Stimme wird dreimal lauter.\b • Du lässt einen unverschlossenen Tür oder ein Fenster aufspringen. \b• Du lässt eine Flamme aufleuchten, erlöschen oder ihre Farbe ändern. \b• Du lässt den Boden für eine Minute beben.\b • Du erzeugst Geräusche, die von dir ausgehen, wie das Flüstern einer Stimme, das Brüllen eines Löwen, das Knurren von Donner oder das Klappern von Metall. \b• Du lässt eine Tür oder ein Fenster, das du berührst, für eine Minute verriegeln oder entriegeln.',
      school: 'Transmutation',
      isRanged: true,
      range: 30,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      id: 3,
      german: 'Beschwörung',
      english: 'conjuration',
      level: 0,
      isCustom: false,
      classes: ['Cleric'],
      timeToCast: 0,
      cost: 'bonus action',
      duration: 10,
      canRitual: 'false',
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: true,
      needsConcentration: true,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Du schützt eine Kreatur in Reichweite vor Angriffen. Während der Wirkungsdauer muss jede Kreatur, die bei einem Angriff oder schädlichen Zauber die geschützte Kreatur als Ziel hat, zuerst einen Weisheitsrettungswurf bestehen. Scheitert der Wurf, muss die Kreatur ein neues Ziel wählen, oder sie verliert den Angriff oder Zauber. Dieser Zauber schützt die entsprechende Kreatur nicht vor Flächeneffekten wie der Explosion eines Feuerballs. Greift die geschützte Kreatur an oder wirkt sie einen Zauber auf eine feindliche Kreatur, endet der Effekt dieses Zaubers.',
      school: 'Conjuration',
      isRanged: true,
      range: 30,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    // Level 1
    {
      id: 1000,
      german: 'Wunden verursachen',
      english: 'inflictWounds',
      level: 1,
      cost: 'action',
      isCustom: false,
      classes: ['Cleric'],
      duration: 0,
      timeToCast: 0,
      canRitual: 'false',
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: false,
      needsAttackRoll: true,
      needsSavingThrow: false,
      doesDamage: true,
      damage: [{ diceNumber: '3', diceType: 'd10', damageType: 'necrotic' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Mach einen Nahkampf-Zauberangriff gegen eine Kreatur deiner Wahl. Bei einem Erfolg erleidet das Ziel 3W10 Nekrotischen Schaden. Auf höheren Graden: Wirkst du diesen Zauber, indem du einen Zauberplatz des 2. Grades oder höher nutzt, steigt der Schaden für jeden Grad über dem 1. um 1W10.',
      school: 'Necromancy',
      isRanged: false,
      range: 5,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      id: 1001,
      german: 'Heilende Berührung',
      english: 'cureWounds',
      level: 1,
      cost: 'action',
      duration: 0,
      isCustom: false,
      classes: ['Bard', 'Cleric', 'Druid', 'Paladin', 'Ranger'],
      timeToCast: 0,
      canRitual: 'false',
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: false,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: true,
      heal: { diceNumber: '1', diceType: 'd8', additionalHeal: 4 },
      description:
        'Eine Kreatur, die du berührst, gewinnt Trefferpunkte in Höhe von 1W8 + deinem Attributsmodifikator im Zauberwirken zurück. Dieser Zauber wirkt nicht auf Untote oder Konstrukte. Auf höheren Graden: Wirkst du diesen Zauber, indem du einen Zauberplatz des 2. Grades oder höher nutzt, steigt die Heilung für jeden Grad über dem 1. um 1W8.',
      school: 'Evocation',
      isRanged: false,
      range: 5,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      id: 1002,
      german: 'Magie entdecken',
      english: 'detectMagic',
      level: 1,
      cost: 'action',
      duration: 100,
      isCustom: false,
      classes: [
        'Bard',
        'Cleric',
        'Druid',
        'Paladin',
        'Ranger',
        'Sorcerer',
        'Wizard',
      ],
      timeToCast: 0,
      canRitual: 'true',
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: true,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Während der Wirkungsdauer nimmst du die Gegenwart von Magie im Abstand von bis zu neun Metern von dir wahr. Wenn du auf diese Weise Magie wahrnimmst, kannst du deine Aktion verwenden, um schwache Auren um sichtbare magische Kreaturen oder Objekte sowie ihre magische Schule zu erkennen, falls vorhanden. Dieser Zauber durchdringt die meisten Barrieren, wird aber von 30 Zentimetern Stein, 2,5 Zentimetern gewöhnlichem Metall, dünnem Bleiblech sowie von einem Meter Holz oder Erde blockiert.',
      school: 'Divination',
      isRanged: true,
      range: 30,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      id: 1003,
      german: 'Segnen',
      english: 'bless',
      level: 1,
      cost: 'action',
      duration: 10,
      isCustom: false,
      classes: ['Cleric', 'Paladin'],
      timeToCast: 0,
      canRitual: 'false',
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: true,
      needsConcentration: true,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Du segnest bis zu drei Kreaturen deiner Wahl in Reichweite. Wenn ein Ziel während der Wirkungsdauer einen Angriffs- oder Rettungswurf ausführt, darf es mit einem W4 würfeln und das Würfelergebnis zu seinem Angriffs- oder Rettungswurf addieren. Auf höheren Graden: Wirkst du diesen Zauber, indem du einen Zauberplatz des 2. Grades oder höher nutzt, kannst du für jeden Grad über dem 1. eine zusätzliche Kreatur als Ziel wählen.',
      school: 'Enchantment',
      isRanged: true,
      range: 60,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      id: 1004,
      german: 'Heilendes Wort',
      english: 'healingWord',
      isCustom: false,
      classes: ['Bard', 'Cleric', 'Druid'],
      level: 1,
      cost: 'bonus action',
      duration: 0,
      timeToCast: 0,
      canRitual: 'false',
      needsVerbal: true,
      needsSomatic: false,
      needsMaterial: false,
      needsConcentration: false,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: true,
      heal: { diceNumber: '1', diceType: 'd4', additionalHeal: 4 },
      description:
        'Eine Kreatur deiner Wahl in Reichweite, die du sehen kannst, gewinnt Trefferpunkte in Höhe von 1W4 + deinem Zauberwirken-Attributsmodifikator zurück. Dieser Zauber wirkt nicht auf Untote oder Konstrukte. Auf höheren Graden: Wirkst du diesen Zauber, indem du einen Zauberplatz des 2. Grades oder höher nutzt, steigt die Heilung für jeden Grad über dem 1. um 1W4.',
      school: 'Evocation',
      isRanged: true,
      range: 60,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      id: 1005,
      german: 'Lenkendes Geschoss',
      english: 'guidingBolt',
      level: 1,
      cost: 'action',
      duration: 0,
      isCustom: false,
      classes: ['Cleric'],
      timeToCast: 0,
      canRitual: 'false',
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: false,
      needsAttackRoll: true,
      needsSavingThrow: false,
      doesDamage: true,
      damage: [{ diceNumber: '4', diceType: 'd6', damageType: 'radiant' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Ein Lichtblitz schießt auf eine Kreatur deiner Wahl in Reichweite zu. Führe einen Fernkampf-Zauberangriff gegen das Ziel aus. Bei einem Erfolg erleidet das Ziel 4W6 gleißenden Schaden. Dank des mystischen dämmrigen Lichts, das auf dem Ziel glitzert, ist der nächste Angriffswurf gegen das Ziel vor Ende deines nächsten Zugs im Vorteil. Auf höheren Graden: Wirkst du diesen Zauber, indem du einen Zauberplatz des 2. Grades oder höher nutzt, steigt der Schaden für jeden Grad über dem 1. um 1W6.',
      school: 'Evocation',
      isRanged: true,
      range: 120,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      id: 1006,
      german: 'Gift und Krankheiten entdecken',
      english: 'detectPoisonAndDisease',
      level: 1,
      cost: 'action',
      duration: 100,
      isCustom: false,
      classes: ['Cleric', 'Druid', 'Paladin', 'Ranger'],
      timeToCast: 0,
      canRitual: 'true',
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: true,
      needsConcentration: true,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Während der Wirkungsdauer nimmst du die Gegenwart und die Position von Giften, giftigen Kreaturen und Krankheiten im Abstand von bis zu neun Metern von dir wahr. Du kannst auch die Art des Gifts, der giftigen Kreatur oder der Krankheit bestimmen. Dieser Zauber durchdringt die meisten Barrieren, wird aber von 30 Zentimetern Stein, 2,5 Zentimetern gewöhnlichem Metall, dünnem Bleiblech sowie von einem Meter Holz oder Erde blockiert.',
      school: 'Divination',
      isRanged: true,
      range: 30,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      id: 1007,
      german: 'Heldenmut',
      english: 'heroism',
      level: 1,
      cost: 'action',
      duration: 10,
      isCustom: false,
      classes: ['Bard', 'Paladin'],
      timeToCast: 0,
      canRitual: 'false',
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: true,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Du berührst eine bereitwillige Kreatur und erfüllst sie mit Tapferkeit. Bis der Zauber endet, kann die Kreatur nicht verängstigt werden, und sie erhält zu Beginn jedes ihrer Züge temporäre Trefferpunkte in Höhe deines Zauberwirken-Attributsmodifikators. Wenn der Zauber endet, verliert das Ziel alle verbleibenden temporären Trefferpunkte dieses Zaubers. Auf höheren Graden: Wirkst du diesen Zauber, indem du einen Zauberplatz des 2. Grades oder höher nutzt, kannst du für jeden Grad über dem 1. eine zusätzliche Kreatur als Ziel wählen.',
      school: 'Enchantment',
      isRanged: false,
      range: 5,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      id: 1008,
      german: 'Heiligtum',
      english: 'sanctuary',
      level: 1,
      isCustom: false,
      classes: ['Cleric'],
      timeToCast: 0,
      cost: 'bonus action',
      duration: 10,
      canRitual: 'false',
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: true,
      needsConcentration: true,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Du schützt eine Kreatur in Reichweite vor Angriffen. Während der Wirkungsdauer muss jede Kreatur, die bei einem Angriff oder schädlichen Zauber die geschützte Kreatur als Ziel hat, zuerst einen Weisheitsrettungswurf bestehen. Scheitert der Wurf, muss die Kreatur ein neues Ziel wählen, oder sie verliert den Angriff oder Zauber. Dieser Zauber schützt die entsprechende Kreatur nicht vor Flächeneffekten wie der Explosion eines Feuerballs. Greift die geschützte Kreatur an oder wirkt sie einen Zauber auf eine feindliche Kreatur, endet der Effekt dieses Zaubers.',
      school: 'Abjuration',
      isRanged: true,
      range: 30,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      id: 1009,
      german: 'Illusion',
      english: 'illusion',
      level: 1,
      isCustom: false,
      classes: ['Cleric'],
      timeToCast: 0,
      cost: 'bonus action',
      duration: 10,
      canRitual: 'false',
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: true,
      needsConcentration: true,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Du schützt eine Kreatur in Reichweite vor Angriffen. Während der Wirkungsdauer muss jede Kreatur, die bei einem Angriff oder schädlichen Zauber die geschützte Kreatur als Ziel hat, zuerst einen Weisheitsrettungswurf bestehen. Scheitert der Wurf, muss die Kreatur ein neues Ziel wählen, oder sie verliert den Angriff oder Zauber. Dieser Zauber schützt die entsprechende Kreatur nicht vor Flächeneffekten wie der Explosion eines Feuerballs. Greift die geschützte Kreatur an oder wirkt sie einen Zauber auf eine feindliche Kreatur, endet der Effekt dieses Zaubers.',
      school: 'Illusion',
      isRanged: true,
      range: 30,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
  ];
}
